% This program is part of the implementation of the Energy-Space Master Equation
% It effectively calculates the energy distribution of charges under applied electric field
% and is valid within the the scope described in - 
% Y. Preezant and N. Tessler, "Carrier heating in disordered organic semiconductors," 
% Physical Review B, vol. 74, p. 235202, Dec 2006


clear all  % erase all variables left from previous runs

global g y_start freq_ep freq_hp freq_ph freq_pph freq_ee freq_hh freq_pe freq_ppe Gen_const Rec_const Ve Vh e_in h_in E_h



kT=0.026*300/300;% [eV]

Efh=-0.4767;%Fermi energy of hole input flow distribution
Th=300;        %Effective temperature of input flow distribution

Sigma_h=4*0.026 %[eV] Sigma_h - is GAUSSIAN width of HOLES energy distribution. 

a=1e-7;%[cm]-grid constant
Gamma_h=0*-10/a;% [1/cm]-inverse of wave-function overlapping parameter for HOLES (not important here)

aF=1*a*[1e3 3e3 8e3 1.00E+04	4.00E+04 9.00E+04 1.60E+05 2.50E+05 3.60E+05 4.90E+05 6.40E+05 8.10E+05	1.00E+06]; %Field-grid constant product - energetic cost for single jump

n_tot_h=0.0001;% Total concentration of HOLES

%Defining energy spans

N_e=201 % number of points in energy vector
E_h=linspace(-30*kT,20*kT,N_e)'; % energy vector


E(:,1)=E_h;%First column of E-array - HOLES energies

% Defining DOS slots at every energy. DOS slot - is an absolute number of DOS states 
% in some vicinity of defined energy.DOS slots have the same value for all three species 


% % % g=(1/kT0)*exp(E_h./kT0)*(max(E_h)-min(E_h))/(max(size(E_h))-1);% EXPONENTIAL DOS
% % % g=(1/kT0)*exp(E_h./kT0)*(max(E_h)-min(E_h))/1e-4;% EXPONENTIAL DOS - UNNORMALIZED

g=(1/sqrt(2*pi)/Sigma_h)*exp(-E_h.^2/2/Sigma_h^2)*(max(E_h)-min(E_h))/(max(size(E_h))-1);% GAUSSIAN DOS
% % % g=(1/sqrt(2*pi)/Sigma_h)*exp(-E_h.^2/2/Sigma_h^2)*(max(E_h)-min(E_h))/1e-7;% GAUSSIAN DOS-UNNORMALIZED

h_in=g./(1+exp((E(:,1)-Efh)/(kT*Th/300)));% Iniial guess of hole Distibution 

n_tot_h=sum(h_in);  % Total concentration of HOLES


for f=1:max(size(aF))

Electric_Field = aF(f)/a
% % % %%%%%  2D , 4-nearest neighbour case%%%%%%%

% HOLE to EMTY jump rate 
freq_hh=exp(a*Gamma_h)*(min(1,kron(exp(E(:,1)/kT),exp(-(E(:,1)'+aF(f))/kT)))+...
        min(1,kron(exp(E(:,1)/kT),exp(-(E(:,1)'-aF(f))/kT)))+...
        2*min(1,kron(exp(E(:,1)/kT),exp(-(E(:,1)')/kT))));
    
% Calling to MATLAB ODE Solver 

y_start=reshape(h_in ,N_e,1);
ib=y_start;
% % load IB21;
% % ib=IB21';
[T,Y] = ode45(@energy_balance_rate,[0 0.01],ib);
tic
figure(1)
hold off
grid
Curr_Error=1; % this will contain the relative difference between current out of sites and current in
ii=0;
%for i=1:2000
while Curr_Error>1e-5,   % if the relative difference is below 1e-5 it is effectively zero (you can change this to lower values)
    ii=ii+1;
    y_length=max(size(Y(:,1)));
    ib=Y(y_length,:);
    YY(ii,:)=ib;
    [T,Y] = ode45(@energy_balance_rate,[0 5],ib);
    if (mod(ii,10)==0),
        ii
    end
   if (mod(ii,100)==0),
       figure(1)
       plot(E_h,ib(1:201),'b')
       text (0.2, max(ib)*0.7,strcat('Electric Field =  ',num2str(Electric_Field)))
       hold on
      pltcurr
       drawnow
   end
end
toc
y_length=max(size(Y(:,1)));
ib=Y(y_length,:);
iibb(f,:)=ib';
save IB22 iibb -ascii

end
figure(4); plot(E_h,iibb(:,1:201))

% % % figure(1),plot(E(:,1),n(1:N_e,:))
% % % figure(2),plot(E(:,2),n(N_e+1:2*N_e,:))
% % % figure(3),plot(E(:,3),n(2*N_e+1:3*N_e,:))
